package gregapi.oredict;

import gregapi.code.ArrayListNoNulls;
import gregapi.data.MT;

import java.util.List;

import net.minecraft.nbt.NBTTagCompound;

/**
 * @author Gregorius Techneticies
 */
public final class OreDictMaterialStack implements Cloneable {
	public long mAmount;
	public OreDictMaterial mMaterial;
	
	public OreDictMaterialStack(OreDictMaterial aMaterial, long aAmount) {
		mMaterial = aMaterial==null?MT.NULL:aMaterial;
		mAmount = aAmount;
	}
	
	public OreDictMaterialStack copy(long aAmount) {
		return new OreDictMaterialStack(mMaterial, aAmount);
	}
	
	@Override
	public OreDictMaterialStack clone() {
		return new OreDictMaterialStack(mMaterial, mAmount);
	}
	
	@Override
	public boolean equals(Object aObject) {
		if (aObject == this) return true;
		if (aObject == null) return false;
		if (aObject instanceof OreDictMaterial) return aObject == mMaterial;
		if (aObject instanceof OreDictMaterialStack) return ((OreDictMaterialStack)aObject).mMaterial == mMaterial && (mAmount < 0 || ((OreDictMaterialStack)aObject).mAmount < 0 || ((OreDictMaterialStack)aObject).mAmount == mAmount);
		return false;
	}
	
	@Override
	public String toString() {
		return mMaterial.toString() + " - " + mAmount;
	}
	
	@Override
	public int hashCode() {
		return mMaterial.hashCode();
	}
	
	public List<OreDictMaterialStack> addToList(List<OreDictMaterialStack> aList) {
		if (mAmount == 0 || mMaterial == null) return aList;
		for (OreDictMaterialStack tMaterial : aList) if (tMaterial.mMaterial == mMaterial) {tMaterial.mAmount += mAmount; return aList;}
		aList.add(clone());
		return aList;
	}
	
	public NBTTagCompound save() {
		NBTTagCompound rNBT = new NBTTagCompound();
		rNBT.setString("m", mMaterial.mNameInternal);
		rNBT.setLong("a", mAmount);
		return rNBT;
	}
	
	public void save(String aTagName, NBTTagCompound aNBT) {
		aNBT.setTag(aTagName, save());
	}
	
	public static OreDictMaterialStack load(NBTTagCompound aNBT) {
		return new OreDictMaterialStack(OreDictMaterial.get(aNBT.getString("m")), aNBT.getLong("a"));
	}
	
	public static OreDictMaterialStack load(String aTagName, NBTTagCompound aNBT) {
		return load(aNBT.getCompoundTag(aTagName));
	}
	
	public static NBTTagCompound saveList(List<OreDictMaterialStack> aList) {
		NBTTagCompound rNBT = new NBTTagCompound();
		if (aList == null) return rNBT;
		int l = 0;
		for (int i = 0, j = aList.size(); i < j; i++) {
			OreDictMaterialStack tStack = aList.get(i);
			if (tStack != null && tStack.mMaterial != MT.NULL) {
				rNBT.setTag(""+i, tStack.save());
				l++;
			}
		}
		rNBT.setInteger("size", l);
		return rNBT;
	}
	
	public static void saveList(String aTagName, NBTTagCompound aNBT, List<OreDictMaterialStack> aList) {
		aNBT.setTag(aTagName, saveList(aList));
	}
	
	public static List<OreDictMaterialStack> loadList(NBTTagCompound aNBT) {
		if (aNBT == null) return null;
		ArrayListNoNulls<OreDictMaterialStack> rList = new ArrayListNoNulls<OreDictMaterialStack>();
		for (int i = 0, j = aNBT.getInteger("size"); i < j; i++) {
			OreDictMaterialStack tStack = load(""+i, aNBT);
			if (tStack.mMaterial != MT.NULL) rList.add(tStack);
		}
		return rList;
	}
	
	public static List<OreDictMaterialStack> loadList(String aTagName, NBTTagCompound aNBT) {
		return loadList(aNBT.getCompoundTag(aTagName));
	}
}